import json

from flask import url_for
from collections import OrderedDict
from uuid import uuid4


PUBLIC_REPO = 'public/publicrepo'

PRIVATE_REPO = 'devtable/shared'

ORG = 'devtableorg'
ORG_REPO = ORG + '/orgrepo'
ORG_READERS = 'readers'
ORG_OWNER = 'devtable'
ORG_OWNERS = 'owners'
ORG_READERS = 'readers'

FAKE_IMAGE_ID = str(uuid4())
FAKE_TAG_NAME = str(uuid4())
FAKE_USERNAME = str(uuid4())
FAKE_TOKEN = str(uuid4())

NEW_ORG_REPO_DETAILS = {
  'repository': str(uuid4()),
  'visibility': 'private',
  'description': '',
  'namespace': ORG,
}

class hashabledict(dict):
  def __hash__(self):
    return hash(tuple(sorted(self.items())))


def open_kwargs(method='GET', json_object=None):
  kwargs = hashabledict([
    ('method', method),
  ])

  if json_object is not None:
    kwargs['data'] = json.dumps(json_object)
    kwargs['content_type'] = 'application/json'

  elif method == 'POST' or method == 'PUT':
    kwargs['data'] = json.dumps({
      'fake': 'json',
      'data': 'here',
    })
    kwargs['content_type'] = 'application/json'

  return kwargs


def build_anon_spec():
  return OrderedDict([
    ((url_for('welcome'), open_kwargs()), 200),

    ((url_for('plans_list'), open_kwargs()), 200),

    ((url_for('get_logged_in_user'), open_kwargs()), 200),

    ((url_for('change_user_details'), open_kwargs('PUT')), 401),

    ((url_for('create_user_api'), open_kwargs('POST')), 400),

    ((url_for('signin_api'), open_kwargs('POST')), 400),

    ((url_for('send_recovery'), open_kwargs('POST')), 400),

    ((url_for('get_matching_users', prefix='dev'), open_kwargs()), 401),

    ((url_for('get_matching_entities', prefix='dev'), open_kwargs()), 401),

    ((url_for('get_organization', orgname=ORG), open_kwargs()), 401),

    ((url_for('get_organization_private_allowed', orgname=ORG),
     open_kwargs()), 401),

    ((url_for('update_organization_team', orgname=ORG, teamname=ORG_OWNERS),
      open_kwargs('PUT')), 401),
    ((url_for('update_organization_team', orgname=ORG, teamname=ORG_READERS),
      open_kwargs('PUT')), 401),

    ((url_for('delete_organization_team', orgname=ORG, teamname=ORG_OWNERS),
      open_kwargs('DELETE')), 401),
    ((url_for('delete_organization_team', orgname=ORG, teamname=ORG_READERS),
      open_kwargs('DELETE')), 401),

    ((url_for('get_organization_team_members', orgname=ORG,
              teamname=ORG_OWNERS), open_kwargs()), 401),
    ((url_for('get_organization_team_members', orgname=ORG,
              teamname=ORG_READERS), open_kwargs()), 401),

    ((url_for('update_organization_team_member', orgname=ORG,
              teamname=ORG_OWNERS, membername=ORG_OWNER),
      open_kwargs('PUT')), 401),
    ((url_for('update_organization_team_member', orgname=ORG,
              teamname=ORG_READERS, membername=ORG_OWNER),
      open_kwargs('PUT')), 401),

    ((url_for('delete_organization_team_member', orgname=ORG,
              teamname=ORG_OWNERS, membername=ORG_OWNER),
      open_kwargs('DELETE')), 401),
    ((url_for('delete_organization_team_member', orgname=ORG,
              teamname=ORG_READERS, membername=ORG_OWNER),
      open_kwargs('DELETE')), 401),

    ((url_for('create_repo_api'), open_kwargs('POST', NEW_ORG_REPO_DETAILS)),
     401),

    ((url_for('match_repos_api'), open_kwargs()), 200),

    ((url_for('list_repos_api'), open_kwargs()), 200),

    ((url_for('update_repo_api', repository=PUBLIC_REPO), open_kwargs('PUT')),
     401),
    ((url_for('update_repo_api', repository=ORG_REPO), open_kwargs('PUT')),
     401),
    ((url_for('update_repo_api', repository=PRIVATE_REPO),
      open_kwargs('PUT')), 401),

    ((url_for('change_repo_visibility_api', repository=PUBLIC_REPO),
      open_kwargs('POST')), 401),
    ((url_for('change_repo_visibility_api', repository=ORG_REPO),
      open_kwargs('POST')), 401),
    ((url_for('change_repo_visibility_api', repository=PRIVATE_REPO),
      open_kwargs('POST')), 401),

    ((url_for('delete_repository', repository=PUBLIC_REPO),
      open_kwargs('DELETE')), 401),
    ((url_for('delete_repository', repository=ORG_REPO),
      open_kwargs('DELETE')), 401),
    ((url_for('delete_repository', repository=PRIVATE_REPO),
      open_kwargs('DELETE')), 401),

    ((url_for('get_repo_api', repository=PUBLIC_REPO), open_kwargs()), 200),
    ((url_for('get_repo_api', repository=ORG_REPO), open_kwargs()), 403),
    ((url_for('get_repo_api', repository=PRIVATE_REPO), open_kwargs()), 403),

    ((url_for('get_repo_builds', repository=PUBLIC_REPO), open_kwargs()),
     401),
    ((url_for('get_repo_builds', repository=ORG_REPO), open_kwargs()), 401),
    ((url_for('get_repo_builds', repository=PRIVATE_REPO), open_kwargs()),
     401),

    ((url_for('get_filedrop_url'), open_kwargs('POST')), 401),

    ((url_for('request_repo_build', repository=PUBLIC_REPO),
      open_kwargs('POST')), 401),
    ((url_for('request_repo_build', repository=ORG_REPO),
      open_kwargs('POST')), 401),
    ((url_for('request_repo_build', repository=PRIVATE_REPO),
      open_kwargs('POST')), 401),

    ((url_for('list_repository_images', repository=PUBLIC_REPO),
      open_kwargs()), 200),
    ((url_for('list_repository_images', repository=ORG_REPO),
      open_kwargs()), 403),
    ((url_for('list_repository_images', repository=PRIVATE_REPO),
      open_kwargs()), 403),

    ((url_for('get_image', repository=PUBLIC_REPO, image_id=FAKE_IMAGE_ID),
      open_kwargs()), 404),
    ((url_for('get_image', repository=ORG_REPO, image_id=FAKE_IMAGE_ID),
      open_kwargs()), 403),
    ((url_for('get_image', repository=PRIVATE_REPO, image_id=FAKE_IMAGE_ID),
      open_kwargs()), 403),

    ((url_for('get_image_changes', repository=PUBLIC_REPO,
             image_id=FAKE_IMAGE_ID), open_kwargs()), 404),
    ((url_for('get_image_changes', repository=ORG_REPO,
             image_id=FAKE_IMAGE_ID), open_kwargs()), 403),
    ((url_for('get_image_changes', repository=PRIVATE_REPO,
             image_id=FAKE_IMAGE_ID), open_kwargs()), 403),

    ((url_for('list_tag_images', repository=PUBLIC_REPO, tag=FAKE_TAG_NAME),
      open_kwargs()), 404),
    ((url_for('list_tag_images', repository=ORG_REPO, tag=FAKE_TAG_NAME),
      open_kwargs()), 403),
    ((url_for('list_tag_images', repository=PRIVATE_REPO, tag=FAKE_TAG_NAME),
      open_kwargs()), 403),
 
    ((url_for('list_repo_team_permissions', repository=PUBLIC_REPO),
      open_kwargs()), 401),
    ((url_for('list_repo_team_permissions', repository=ORG_REPO),
      open_kwargs()), 401),
    ((url_for('list_repo_team_permissions', repository=PRIVATE_REPO),
      open_kwargs()), 401),

    ((url_for('list_repo_user_permissions', repository=PUBLIC_REPO),
      open_kwargs()), 401),
    ((url_for('list_repo_user_permissions', repository=ORG_REPO),
      open_kwargs()), 401),
    ((url_for('list_repo_user_permissions', repository=PRIVATE_REPO),
      open_kwargs()), 401),

    ((url_for('get_user_permissions', repository=PUBLIC_REPO,
             username=FAKE_USERNAME), open_kwargs()), 401),
    ((url_for('get_user_permissions', repository=ORG_REPO,
             username=FAKE_USERNAME), open_kwargs()), 401),
    ((url_for('get_user_permissions', repository=PRIVATE_REPO,
             username=FAKE_USERNAME), open_kwargs()), 401),

    ((url_for('get_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_OWNERS), open_kwargs()), 401),
    ((url_for('get_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_READERS), open_kwargs()), 401),
    ((url_for('get_team_permissions', repository=ORG_REPO,
             teamname=ORG_OWNERS), open_kwargs()), 401),
    ((url_for('get_team_permissions', repository=ORG_REPO,
             teamname=ORG_READERS), open_kwargs()), 401),
    ((url_for('get_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_OWNERS), open_kwargs()), 401),
    ((url_for('get_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_READERS), open_kwargs()), 401),

    ((url_for('change_user_permissions', repository=PUBLIC_REPO,
             username=FAKE_USERNAME), open_kwargs('PUT')), 401),
    ((url_for('change_user_permissions', repository=ORG_REPO,
             username=FAKE_USERNAME), open_kwargs('PUT')), 401),
    ((url_for('change_user_permissions', repository=PRIVATE_REPO,
             username=FAKE_USERNAME), open_kwargs('PUT')), 401),

    ((url_for('change_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_OWNERS), open_kwargs('PUT')), 401),
    ((url_for('change_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_READERS), open_kwargs('PUT')), 401),
    ((url_for('change_team_permissions', repository=ORG_REPO,
             teamname=ORG_OWNERS), open_kwargs('PUT')), 401),
    ((url_for('change_team_permissions', repository=ORG_REPO,
             teamname=ORG_READERS), open_kwargs('PUT')), 401),
    ((url_for('change_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_OWNERS), open_kwargs('PUT')), 401),
    ((url_for('change_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_READERS), open_kwargs('PUT')), 401),

    ((url_for('delete_user_permissions', repository=PUBLIC_REPO,
             username=FAKE_USERNAME), open_kwargs('DELETE')), 401),
    ((url_for('delete_user_permissions', repository=ORG_REPO,
             username=FAKE_USERNAME), open_kwargs('DELETE')), 401),
    ((url_for('delete_user_permissions', repository=PRIVATE_REPO,
             username=FAKE_USERNAME), open_kwargs('DELETE')), 401),

    ((url_for('delete_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_OWNERS), open_kwargs('DELETE')), 401),
    ((url_for('delete_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_READERS), open_kwargs('DELETE')), 401),
    ((url_for('delete_team_permissions', repository=ORG_REPO,
             teamname=ORG_OWNERS), open_kwargs('DELETE')), 401),
    ((url_for('delete_team_permissions', repository=ORG_REPO,
             teamname=ORG_READERS), open_kwargs('DELETE')), 401),
    ((url_for('delete_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_OWNERS), open_kwargs('DELETE')), 401),
    ((url_for('delete_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_READERS), open_kwargs('DELETE')), 401),

    ((url_for('list_repo_tokens', repository=PUBLIC_REPO), open_kwargs()),
     401),
    ((url_for('list_repo_tokens', repository=ORG_REPO), open_kwargs()), 401),
    ((url_for('list_repo_tokens', repository=PRIVATE_REPO), open_kwargs()),
     401),

    ((url_for('get_tokens', repository=PUBLIC_REPO, code=FAKE_TOKEN),
      open_kwargs()), 401),
    ((url_for('get_tokens', repository=ORG_REPO, code=FAKE_TOKEN),
      open_kwargs()), 401),
    ((url_for('get_tokens', repository=PRIVATE_REPO, code=FAKE_TOKEN),
      open_kwargs()), 401),

    ((url_for('create_token', repository=PUBLIC_REPO), open_kwargs('POST')),
     401),
    ((url_for('create_token', repository=ORG_REPO), open_kwargs('POST')),
     401),
    ((url_for('create_token', repository=PRIVATE_REPO), open_kwargs('POST')),
     401),

    ((url_for('change_token', repository=PUBLIC_REPO, code=FAKE_TOKEN), 
      open_kwargs('PUT')), 401),
    ((url_for('change_token', repository=ORG_REPO, code=FAKE_TOKEN), 
      open_kwargs('PUT')), 401),
    ((url_for('change_token', repository=PRIVATE_REPO, code=FAKE_TOKEN), 
      open_kwargs('PUT')), 401),

    ((url_for('delete_token', repository=PUBLIC_REPO, code=FAKE_TOKEN), 
      open_kwargs('DELETE')), 401),
    ((url_for('delete_token', repository=ORG_REPO, code=FAKE_TOKEN), 
      open_kwargs('DELETE')), 401),
    ((url_for('delete_token', repository=PRIVATE_REPO, code=FAKE_TOKEN), 
      open_kwargs('DELETE')), 401),

    ((url_for('subscribe_api'), open_kwargs('PUT')), 401),

    ((url_for('subscribe_org_api', orgname=ORG), open_kwargs('PUT')), 401),

    ((url_for('get_subscription'), open_kwargs()), 401),

    ((url_for('get_org_subscription', orgname=ORG), open_kwargs()), 401),
  ])


def build_no_access_spec():
  changes = OrderedDict([
    ((url_for('change_user_details'), open_kwargs('PUT')), 200),

    ((url_for('get_matching_users', prefix='dev'), open_kwargs()), 200),

    ((url_for('get_matching_entities', prefix='dev'), open_kwargs()), 200),

    ((url_for('get_organization', orgname=ORG), open_kwargs()), 403),

    ((url_for('get_organization_private_allowed', orgname=ORG),
     open_kwargs()), 403),

    ((url_for('update_organization_team', orgname=ORG, teamname=ORG_OWNERS),
      open_kwargs('PUT')), 403),
    ((url_for('update_organization_team', orgname=ORG, teamname=ORG_READERS),
      open_kwargs('PUT')), 403),

    ((url_for('delete_organization_team', orgname=ORG, teamname=ORG_OWNERS),
      open_kwargs('DELETE')), 403),
    ((url_for('delete_organization_team', orgname=ORG, teamname=ORG_READERS),
      open_kwargs('DELETE')), 403),

    ((url_for('get_organization_team_members', orgname=ORG,
              teamname=ORG_OWNERS), open_kwargs()), 403),
    ((url_for('get_organization_team_members', orgname=ORG,
              teamname=ORG_READERS), open_kwargs()), 403),

    ((url_for('update_organization_team_member', orgname=ORG,
              teamname=ORG_OWNERS, membername=ORG_OWNER),
      open_kwargs('PUT')), 403),
    ((url_for('update_organization_team_member', orgname=ORG,
              teamname=ORG_READERS, membername=ORG_OWNER),
      open_kwargs('PUT')), 403),

    ((url_for('delete_organization_team_member', orgname=ORG,
              teamname=ORG_OWNERS, membername=ORG_OWNER),
      open_kwargs('DELETE')), 403),
    ((url_for('delete_organization_team_member', orgname=ORG,
              teamname=ORG_READERS, membername=ORG_OWNER),
      open_kwargs('DELETE')), 403),

    ((url_for('create_repo_api'), open_kwargs('POST', NEW_ORG_REPO_DETAILS)),
     403),

    ((url_for('update_repo_api', repository=PUBLIC_REPO), open_kwargs('PUT')),
     403),
    ((url_for('update_repo_api', repository=ORG_REPO), open_kwargs('PUT')),
     403),
    ((url_for('update_repo_api', repository=PRIVATE_REPO),
      open_kwargs('PUT')), 403),

    ((url_for('change_repo_visibility_api', repository=PUBLIC_REPO),
      open_kwargs('POST')), 403),
    ((url_for('change_repo_visibility_api', repository=ORG_REPO),
      open_kwargs('POST')), 403),
    ((url_for('change_repo_visibility_api', repository=PRIVATE_REPO),
      open_kwargs('POST')), 403),

    ((url_for('delete_repository', repository=PUBLIC_REPO),
      open_kwargs('DELETE')), 403),
    ((url_for('delete_repository', repository=ORG_REPO),
      open_kwargs('DELETE')), 403),
    ((url_for('delete_repository', repository=PRIVATE_REPO),
      open_kwargs('DELETE')), 403),

    ((url_for('get_repo_builds', repository=PUBLIC_REPO), open_kwargs()),
     403),
    ((url_for('get_repo_builds', repository=ORG_REPO), open_kwargs()), 403),
    ((url_for('get_repo_builds', repository=PRIVATE_REPO), open_kwargs()),
     403),

    ((url_for('get_filedrop_url'), open_kwargs('POST')), 400),

    ((url_for('request_repo_build', repository=PUBLIC_REPO),
      open_kwargs('POST')), 403),
    ((url_for('request_repo_build', repository=ORG_REPO),
      open_kwargs('POST')), 403),
    ((url_for('request_repo_build', repository=PRIVATE_REPO),
      open_kwargs('POST')), 403),

    ((url_for('list_repo_team_permissions', repository=PUBLIC_REPO),
      open_kwargs()), 403),
    ((url_for('list_repo_team_permissions', repository=ORG_REPO),
      open_kwargs()), 403),
    ((url_for('list_repo_team_permissions', repository=PRIVATE_REPO),
      open_kwargs()), 403),

    ((url_for('list_repo_user_permissions', repository=PUBLIC_REPO),
      open_kwargs()), 403),
    ((url_for('list_repo_user_permissions', repository=ORG_REPO),
      open_kwargs()), 403),
    ((url_for('list_repo_user_permissions', repository=PRIVATE_REPO),
      open_kwargs()), 403),

    ((url_for('get_user_permissions', repository=PUBLIC_REPO,
             username=FAKE_USERNAME), open_kwargs()), 403),
    ((url_for('get_user_permissions', repository=ORG_REPO,
             username=FAKE_USERNAME), open_kwargs()), 403),
    ((url_for('get_user_permissions', repository=PRIVATE_REPO,
             username=FAKE_USERNAME), open_kwargs()), 403),

    ((url_for('get_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_OWNERS), open_kwargs()), 403),
    ((url_for('get_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_READERS), open_kwargs()), 403),
    ((url_for('get_team_permissions', repository=ORG_REPO,
             teamname=ORG_OWNERS), open_kwargs()), 403),
    ((url_for('get_team_permissions', repository=ORG_REPO,
             teamname=ORG_READERS), open_kwargs()), 403),
    ((url_for('get_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_OWNERS), open_kwargs()), 403),
    ((url_for('get_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_READERS), open_kwargs()), 403),

    ((url_for('change_user_permissions', repository=PUBLIC_REPO,
             username=FAKE_USERNAME), open_kwargs('PUT')), 403),
    ((url_for('change_user_permissions', repository=ORG_REPO,
             username=FAKE_USERNAME), open_kwargs('PUT')), 403),
    ((url_for('change_user_permissions', repository=PRIVATE_REPO,
             username=FAKE_USERNAME), open_kwargs('PUT')), 403),

    ((url_for('change_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_OWNERS), open_kwargs('PUT')), 403),
    ((url_for('change_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_READERS), open_kwargs('PUT')), 403),
    ((url_for('change_team_permissions', repository=ORG_REPO,
             teamname=ORG_OWNERS), open_kwargs('PUT')), 403),
    ((url_for('change_team_permissions', repository=ORG_REPO,
             teamname=ORG_READERS), open_kwargs('PUT')), 403),
    ((url_for('change_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_OWNERS), open_kwargs('PUT')), 403),
    ((url_for('change_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_READERS), open_kwargs('PUT')), 403),

    ((url_for('delete_user_permissions', repository=PUBLIC_REPO,
             username=FAKE_USERNAME), open_kwargs('DELETE')), 403),
    ((url_for('delete_user_permissions', repository=ORG_REPO,
             username=FAKE_USERNAME), open_kwargs('DELETE')), 403),
    ((url_for('delete_user_permissions', repository=PRIVATE_REPO,
             username=FAKE_USERNAME), open_kwargs('DELETE')), 403),

    ((url_for('delete_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_OWNERS), open_kwargs('DELETE')), 403),
    ((url_for('delete_team_permissions', repository=PUBLIC_REPO,
             teamname=ORG_READERS), open_kwargs('DELETE')), 403),
    ((url_for('delete_team_permissions', repository=ORG_REPO,
             teamname=ORG_OWNERS), open_kwargs('DELETE')), 403),
    ((url_for('delete_team_permissions', repository=ORG_REPO,
             teamname=ORG_READERS), open_kwargs('DELETE')), 403),
    ((url_for('delete_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_OWNERS), open_kwargs('DELETE')), 403),
    ((url_for('delete_team_permissions', repository=PRIVATE_REPO,
             teamname=ORG_READERS), open_kwargs('DELETE')), 403),

    ((url_for('list_repo_tokens', repository=PUBLIC_REPO), open_kwargs()),
     403),
    ((url_for('list_repo_tokens', repository=ORG_REPO), open_kwargs()), 403),
    ((url_for('list_repo_tokens', repository=PRIVATE_REPO), open_kwargs()),
     403),

    ((url_for('get_tokens', repository=PUBLIC_REPO, code=FAKE_TOKEN),
      open_kwargs()), 403),
    ((url_for('get_tokens', repository=ORG_REPO, code=FAKE_TOKEN),
      open_kwargs()), 403),
    ((url_for('get_tokens', repository=PRIVATE_REPO, code=FAKE_TOKEN),
      open_kwargs()), 403),

    ((url_for('create_token', repository=PUBLIC_REPO), open_kwargs('POST')),
     403),
    ((url_for('create_token', repository=ORG_REPO), open_kwargs('POST')),
     403),
    ((url_for('create_token', repository=PRIVATE_REPO), open_kwargs('POST')),
     403),

    ((url_for('change_token', repository=PUBLIC_REPO, code=FAKE_TOKEN), 
      open_kwargs('PUT')), 403),
    ((url_for('change_token', repository=ORG_REPO, code=FAKE_TOKEN), 
      open_kwargs('PUT')), 403),
    ((url_for('change_token', repository=PRIVATE_REPO, code=FAKE_TOKEN), 
      open_kwargs('PUT')), 403),

    ((url_for('delete_token', repository=PUBLIC_REPO, code=FAKE_TOKEN), 
      open_kwargs('DELETE')), 403),
    ((url_for('delete_token', repository=ORG_REPO, code=FAKE_TOKEN), 
      open_kwargs('DELETE')), 403),
    ((url_for('delete_token', repository=PRIVATE_REPO, code=FAKE_TOKEN), 
      open_kwargs('DELETE')), 403),

    ((url_for('subscribe_api'), open_kwargs('PUT')), 400),

    ((url_for('subscribe_org_api', orgname=ORG), open_kwargs('PUT')), 403),

    ((url_for('get_subscription'), open_kwargs()), 200),

    ((url_for('get_org_subscription', orgname=ORG), open_kwargs()), 403),
  ])

  to_update = build_anon_spec()
  to_update.update(changes)
  return to_update


def build_read_access_spec():
  changes = OrderedDict([
    ((url_for('get_organization', orgname=ORG), open_kwargs()), 200),

    ((url_for('get_organization_team_members', orgname=ORG,
              teamname=ORG_READERS), open_kwargs()), 200),

    ((url_for('create_repo_api'), open_kwargs('POST', NEW_ORG_REPO_DETAILS)),
     403),

    ((url_for('get_repo_api', repository=ORG_REPO), open_kwargs()), 200),
    ((url_for('get_repo_api', repository=PRIVATE_REPO), open_kwargs()), 200),

    ((url_for('list_repository_images', repository=ORG_REPO),
      open_kwargs()), 200),
    ((url_for('list_repository_images', repository=PRIVATE_REPO),
      open_kwargs()), 200),

    ((url_for('get_image', repository=ORG_REPO, image_id=FAKE_IMAGE_ID),
      open_kwargs()), 404),
    ((url_for('get_image', repository=PRIVATE_REPO, image_id=FAKE_IMAGE_ID),
      open_kwargs()), 404),

    ((url_for('get_image_changes', repository=ORG_REPO,
             image_id=FAKE_IMAGE_ID), open_kwargs()), 404),
    ((url_for('get_image_changes', repository=PRIVATE_REPO,
             image_id=FAKE_IMAGE_ID), open_kwargs()), 404),

    ((url_for('list_tag_images', repository=ORG_REPO, tag=FAKE_TAG_NAME),
      open_kwargs()), 404),
    ((url_for('list_tag_images', repository=PRIVATE_REPO, tag=FAKE_TAG_NAME),
      open_kwargs()), 404), 
  ])

  to_update = build_no_access_spec()
  to_update.update(changes)
  return to_update
